#!/usr/bin/env python3
# python atlas/lib/idds/atlas/processing/hyperparameteropt_bayesian.py --max_points 5 --num_points 5 --input main/lib/idds/tests/idds_input.json --output output.json

import argparse
import json
from bayes_opt import BayesianOptimization, UtilityFunction

parser = argparse.ArgumentParser()
parser.add_argument('--max_points', action='store', type=int, required=True, help='max number of points to be generated')
parser.add_argument('--num_points', action='store', type=int, required=True, help='number of points to be generated')
parser.add_argument('--input', action='store', required=True, help='input json file which includes all pre-generated points')
parser.add_argument('--output', action='store', required=True, help='output json file where outputs will be wrote')

args = parser.parse_args()


def get_input_points(input):
    points = None
    opt_space = None
    with open(input) as input_json:
        opt_points = json.load(input_json)
    if 'points' in opt_points:
        points = opt_points['points']
    if 'opt_space' in opt_points:
        opt_space = opt_points['opt_space']
    return points, opt_space


def write_output_points(new_points, output):
    with open(args.output, 'w') as output_json:
        json.dump(new_points, output_json)


def generate_new_points(input_points, opt_space, max_points, num_points):
    # pbounds={'x': (-2, 2), 'y': (-3, 3)}
    if len(input_points) > max_points:
        return []
    num_points = min(num_points, max_points - len(input_points))

    utility = UtilityFunction(kind="ucb", kappa=2.5, xi=0.0)
    optimizer = BayesianOptimization(f=None,
                                     pbounds=opt_space,
                                     verbose=2,
                                     random_state=1)

    unfinished_points = []
    for input_point in input_points:
        point, loss = input_point
        if loss:
            optimizer.register(point, loss)
        else:
            unfinished_points.append(point)

    new_points = []
    for _ in range(num_points):
        x = optimizer.suggest(utility)
        # TODO: check duplication
        if x in unfinished_points:
            continue
        new_points.append(x)
    # recommendation = optimizer.provide_recommendation()
    return new_points


input_points, opt_space = get_input_points(args.input)
new_points = generate_new_points(input_points, opt_space, args.max_points, args.num_points)
write_output_points(new_points, args.output)



#!/usr/bin/env python
#
# Licensed under the Apache License, Version 2.0 (the "License");
# You may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0OA
#
# Authors:
# - Wen Guan, <wen.guan@cern.ch>, 2019


"""
Class of rule creator plubin
"""

import traceback

from rucio.common.exception import DuplicateRule

from idds.atlas.rucio.base_plugin import RucioPluginBase


class RuleCreator(RucioPluginBase):
    def __init__(self, lifetime=3600 * 24 * 7, **kwargs):
        super(RuleCreator, self).__init__(**kwargs)
        self.lifetime = int(lifetime)

    def __call__(self, dataset_scope, dataset_name, dids, dest_rse, src_rse=None, lifetime=None):
        try:
            # dids = [{'scope': <scope>, 'name': <name>}]
            if not lifetime:
                lifetime = self.lifetime

            self.client.add_dataset(scope=dataset_scope, name=dataset_name, lifetime=lifetime)
            self.client.attach_dids(scope=dataset_scope, name=dataset_name, dids=dids)
            self.client.set_status(scope=dataset_scope, name=dataset_name, open=False)

            ds_did = {'scope': dataset_scope, 'name': dataset_name}
            rule_id = self.client.add_replication_rule(dids=[ds_did],
                                                       copies=1,
                                                       rse_expression=dest_rse,
                                                       source_replica_expression=src_rse,
                                                       lifetime=lifetime,
                                                       locked=False,
                                                       grouping='DATASET',
                                                       ask_approval=False)
            if type(rule_id) in (list, tuple):
                rule_id = rule_id[0]
            return rule_id
        except DuplicateRule as ex:
            self.logger.warn(ex)
            rules = self.client.list_did_rules(scope=dataset_scope, name=dataset_name)
            for rule in rules:
                if rule['account'] == self.client.account and rule['rse_expression'] == dest_rse:
                    return rule['id']
        except Exception as ex:
            self.logger.error(ex)
            self.logger.error(traceback.format_exc())
            # raise exceptions.AgentPluginError('%s: %s' % (str(ex), traceback.format_exc()))
        return None



#!/usr/bin/env python
#
# Licensed under the Apache License, Version 2.0 (the "License");
# You may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# http://www.apache.org/licenses/LICENSE-2.0OA
#
# Authors:
# - Wen Guan, <wen.guan@cern.ch>, 2019


"""
Class of collection lister plubin
"""

import traceback

from rucio.common.exception import DataIdentifierNotFound

from idds.common import exceptions
from idds.atlas.rucio.base_plugin import RucioPluginBase


class ContentsRegister(RucioPluginBase):
    def __init__(self, **kwargs):
        super(ContentsRegister, self).__init__(**kwargs)

    def __call__(self, coll_scope, coll_name, account=None, lifetime=7 * 24 * 3600, rse=None, files=[]):
        try:
            try:
                self.client.get_did(scope=coll_scope, name=coll_name)
            except DataIdentifierNotFound:
                self.client.add_dataset(scope=coll_scope,
                                        name=coll_name,
                                        rules=[{'account': account,
                                                'copies': 1,
                                                'rse_expression': rse,
                                                'grouping': 'DATASET',
                                                'lifetime': lifetime}])

            new_files = []
            for file in files:
                new_file = {'scope': file['scope'],
                            'name': file['name'],
                            'state': 'A',   # available
                            'bytes': file['content_size'],
                            'adler32': file['adler32'],
                            'pfn': file['pfn']}
                new_files.append(new_file)

            self.client.add_replicas(rse=rse, files=new_files)
            self.client.add_files_to_dataset(scope=coll_scope,
                                             name=coll_name,
                                             files=[new_files],
                                             rse=rse)
        except Exception as error:
            self.logger.error(error)
            self.logger.error(traceback.format_exc())
            raise exceptions.AgentPluginError('%s: %s' % (str(error), traceback.format_exc()))
